# Generated by Django 3.2.13 on 2022-05-15 15:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('redisboard', '0004_cleanup'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='redisserver',
            name='sampling_size',
        ),
        migrations.RemoveField(
            model_name='redisserver',
            name='sampling_threshold',
        ),
    ]
